'****************************************************************************
'
' UKAW Paket   : MD5-Implementationen (MD5, DIGEST-MD5, CRAM-MD5)
' Version      : 3.05g
' Compilation  : Thomas Gohel (gohel@basicguru.com)
' Last Files at: http://www.gohel.de
' Last Updates : see directly in the sources
'
'****************************************************************************
'
' Original code notice:
'
' This code implements the MD5 message-digest algorithm. The algorithm is
' due TO Ron Rivest.  This code was written by Colin Plumb IN 1993, no
' copyright is claimed. This code is IN the public domain; DO WITH it what
' you wish.
'
' Equivalent code is available FROM RSA DATA Security, Inc. This code has
' been tested against that, AND is equivalent, except that you don't need to
' include two pages OF legalese WITH every copy.
'
' TO compute the message digest of a chunk of bytes, DECLARE an MD5Context
' structure, pass it TO MD5Init, CALL MD5Update AS needed ON buffers full
' OF bytes, AND THEN CALL MD5Final, which will fill a supplied 16-BYTE
' ARRAY WITH the digest.
'
' 10.04.2000 - Translated to PowerBASIC/DLL32 by Florent Heyworth
' 10.12.2000 - Translated to PowerBASIC/CC by Thomas Gohel
'              Add support for CRAM-MD5 for SMTP AUTH (RFC 2104)
' 26.12.2001 - Add support for PowerBASIC/DLL v6.10 and PowerBASIC/CC v2.10
' 24.12.2002 - Removed Win32-MemoryAPI
' 28.12.2002 - fixed problem with SASL/DIGEST-MD5 (RFC 2831)
'
'****************************************************************************

$IF NOT %DEF(%NULL)
%NULL = 0
$ENDIF

DECLARE FUNCTION Get_Digest_MD5(BYVAL szText AS STRING) AS STRING
DECLARE FUNCTION Get_String_MD5(BYVAL szText AS STRING) AS STRING
DECLARE FUNCTION Get_CRAM_MD5(BYVAL sText AS STRING, BYVAL sKey AS STRING) AS STRING
DECLARE SUB MD5_MoveMemory (BYVAL pDestination AS DWORD, BYVAL pSource AS DWORD, BYVAL cbLength AS LONG)
DECLARE SUB MD5_FillMemory (BYVAL pDestination AS DWORD, BYVAL cbLength AS LONG, BYVAL bFill AS BYTE)
DECLARE SUB MD5_ZeroMemory (BYVAL pDestination AS DWORD, BYVAL cbLength AS LONG)

'-------------------------------------------------------------------------------------------
' Win32-Memory-API
'-------------------------------------------------------------------------------------------
SUB MD5_MoveMemory (BYVAL pDestination AS DWORD, BYVAL pSource AS DWORD,  BYVAL cbLength AS LONG)

        DIM pDest        AS BYTE PTR          '
        DIM pSour        AS BYTE PTR
        DIM i            AS LOCAL LONG
        DIM bFill        AS LOCAL BYTE

        pDest = pDestination
        pSour = pSource

        FOR i& = 1 TO cbLength
            bFill = @pSour
            @pDest = bFill
            INCR pSour
            INCR pDest
        NEXT i&

END SUB

SUB MD5_FillMemory (BYVAL pDestination AS DWORD, BYVAL cbLength AS LONG, BYVAL bFill AS BYTE)

        DIM i            AS LONG              '
        DIM pDest        AS BYTE PTR
        pDest = pDestination
        FOR i& = 1 TO cbLength
            @pDest = bFill
           INCR pDest
        NEXT i&

END SUB

SUB MD5_ZeroMemory (BYVAL pDestination AS DWORD, BYVAL cbLength AS LONG)

        MD5_FillMemory pDestination, cbLength, 0

END SUB


'-------------------------------------------------------------------------------------------
' Constants FOR MD5_Transform routine
'-------------------------------------------------------------------------------------------

%S11 = 7
%S12 = 12
%S13 = 17
%S14 = 22
%S21 = 5
%S22 = 9
%S23 = 14
%S24 = 20
%S31 = 4
%S32 = 11
%S33 = 16
%S34 = 23
%S41 = 6
%S42 = 10
%S43 = 15
%S44 = 21

TYPE MD5_CTX
        lState(4)    AS LONG         ' state (ABCD)
        lCount(2)    AS LONG         ' number of bits modulo 2^64 (lsp first)
        bBuf         AS STRING * 64  ' input buffer
END TYPE

'-------------------------------------------------------------------------------------------
' PB really should have BUILT-IN function for shift instead of SHIFT LEFT|RIGHT semantics....
'-------------------------------------------------------------------------------------------
FUNCTION MD5_ShiftRight(BYVAL lThis AS LONG, BYVAL lBits AS LONG) AS LONG

        SHIFT RIGHT lThis, lBits
        FUNCTION = lThis

END FUNCTION

FUNCTION MD5_ShiftLeft(BYVAL lThis AS LONG, BYVAL lBits AS LONG) AS LONG

        SHIFT LEFT lThis, lBits
        FUNCTION = lThis

END FUNCTION

'-------------------------------------------------------------------------------------------
'
'-------------------------------------------------------------------------------------------
SUB MD5_Init(BYVAL dwContext AS DWORD)

        'MD5 initialization. Begins an MD5 operation, writing a new context
        LOCAL tContext     AS MD5_CTX PTR '

        tContext = dwContext

        @tContext.lCount(0) = 0
        @tContext.lCount(1) = 0

        ' Load magic initialization constants.
        @tContext.lState(0) = &H67452301
        @tContext.lState(1) = &HEFCDAB89
        @tContext.lState(2) = &H98BADCFE
        @tContext.lState(3) = &H10325476

END SUB

SUB MD5_Transform (BYVAL dwcBuf AS DWORD, BYVAL dwBuf AS DWORD)

        LOCAL a            AS LONG        '
        LOCAL b            AS LONG        '
        LOCAL c            AS LONG        '
        LOCAL d            AS LONG        '
        LOCAL bBuf         AS LONG PTR    '
        LOCAL tContext     AS MD5_CTX PTR

        bBuf = dwBuf
        tContext = dwcBuf
        a = @tContext.lState(0)
        b = @tContext.lState(1)
        c = @tContext.lState(2)
        d = @tContext.lState(3)

        ' ROUND 1
        a = a + ((((b)) AND ((c))) OR ((NOT (b)) AND ((d)))) + (@bBuf[ 0]) +&Hd76aa478
        ROTATE LEFT a, %S11: a = a + b
        d = d + ((((a)) AND ((b))) OR ((NOT (a)) AND ((c)))) + (@bBuf[ 1]) +&He8c7b756
        ROTATE LEFT d, %S12: d = d + a
        c = c + ((((d)) AND ((a))) OR ((NOT (d)) AND ((b)))) + (@bBuf[ 2]) +&H242070db
        ROTATE LEFT c, %S13: c = c + d
        b = b + ((((c)) AND ((d))) OR ((NOT (c)) AND ((a)))) + (@bBuf[ 3]) +&Hc1bdceee
        ROTATE LEFT b, %S14: b = b + c
        '
        a = a + ((((b)) AND ((c))) OR ((NOT (b)) AND ((d)))) + (@bBuf[ 4]) +&Hf57c0faf
        ROTATE LEFT a, %S11: a = a + b
        d = d + ((((a)) AND ((b))) OR ((NOT (a)) AND ((c)))) + (@bBuf[ 5]) +&H4787c62a
        ROTATE LEFT d, %S12: d = d + a
        c = c + ((((d)) AND ((a))) OR ((NOT (d)) AND ((b)))) + (@bBuf[ 6]) +&Ha8304613
        ROTATE LEFT c, %S13: c = c + d
        b = b + ((((c)) AND ((d))) OR ((NOT (c)) AND ((a)))) + (@bBuf[ 7]) +&Hfd469501
        ROTATE LEFT b, %S14: b = b + c
        '
        a = a + ((((b)) AND ((c))) OR ((NOT (b)) AND ((d)))) + (@bBuf[ 8]) +&H698098d8
        ROTATE LEFT a, %S11: a = a + b
        d = d + ((((a)) AND ((b))) OR ((NOT (a)) AND ((c)))) + (@bBuf[ 9]) +&H8b44f7af
        ROTATE LEFT d, %S12: d = d + a
        c = c + ((((d)) AND ((a))) OR ((NOT (d)) AND ((b)))) + (@bBuf[10]) +&Hffff5bb1
        ROTATE LEFT c, %S13: c = c + d
        b = b + ((((c)) AND ((d))) OR ((NOT (c)) AND ((a)))) + (@bBuf[11]) +&H895cd7be
        ROTATE LEFT b, %S14: b = b + c
        '
        a = a + ((((b)) AND ((c))) OR ((NOT (b)) AND ((d)))) + (@bBuf[12]) +&H6b901122
        ROTATE LEFT a, %S11: a = a + b
        d = d + ((((a)) AND ((b))) OR ((NOT (a)) AND ((c)))) + (@bBuf[13]) +&Hfd987193
        ROTATE LEFT d, %S12: d = d + a
        c = c + ((((d)) AND ((a))) OR ((NOT (d)) AND ((b)))) + (@bBuf[14]) +&Ha679438e
        ROTATE LEFT c, %S13: c = c + d
        b = b + ((((c)) AND ((d))) OR ((NOT (c)) AND ((a)))) + (@bBuf[15]) +&H49b40821
        ROTATE LEFT b, %S14: b = b + c

        ' ROUND 2
        a = a + ((((b)) AND ((d))) OR (((c)) AND (NOT (d)))) + (@bBuf[ 1]) +&Hf61e2562
        ROTATE LEFT a, %S21: a = a + b
        d = d + ((((a)) AND ((c))) OR (((b)) AND (NOT (c)))) + (@bBuf[ 6]) +&Hc040b340
        ROTATE LEFT d, %S22: d = d + a
        c = c + ((((d)) AND ((b))) OR (((a)) AND (NOT (b)))) + (@bBuf[11]) +&H265e5a51
        ROTATE LEFT c, %S23: c = c + d
        b = b + ((((c)) AND ((a))) OR (((d)) AND (NOT (a)))) + (@bBuf[ 0]) +&He9b6c7aa
        ROTATE LEFT b, %S24: b = b + c
        '
        a = a + ((((b)) AND ((d))) OR (((c)) AND (NOT (d)))) + (@bBuf[ 5]) +&Hd62f105d
        ROTATE LEFT a, %S21: a = a + b
        d = d + ((((a)) AND ((c))) OR (((b)) AND (NOT (c)))) + (@bBuf[10]) +&H2441453
        ROTATE LEFT d, %S22: d = d + a
        c = c + ((((d)) AND ((b))) OR (((a)) AND (NOT (b)))) + (@bBuf[15]) +&Hd8a1e681
        ROTATE LEFT c, %S23: c = c + d
        b = b + ((((c)) AND ((a))) OR (((d)) AND (NOT (a)))) + (@bBuf[ 4]) +&He7d3fbc8
        ROTATE LEFT b, %S24: b = b + c
        '
        a = a + ((((b)) AND ((d))) OR (((c)) AND (NOT (d)))) + (@bBuf[ 9]) +&H21e1cde6
        ROTATE LEFT a, %S21: a = a + b
        d = d + ((((a)) AND ((c))) OR (((b)) AND (NOT (c)))) + (@bBuf[14]) +&Hc33707d6
        ROTATE LEFT d, %S22: d = d + a
        c = c + ((((d)) AND ((b))) OR (((a)) AND (NOT (b)))) + (@bBuf[ 3]) +&Hf4d50d87
        ROTATE LEFT c, %S23: c = c + d
        b = b + ((((c)) AND ((a))) OR (((d)) AND (NOT (a)))) + (@bBuf[ 8]) +&H455a14ed
        ROTATE LEFT b, %S24: b = b + c
        '
        a = a + ((((b)) AND ((d))) OR (((c)) AND (NOT (d)))) + (@bBuf[13]) +&Ha9e3e905
        ROTATE LEFT a, %S21: a = a + b
        d = d + ((((a)) AND ((c))) OR (((b)) AND (NOT (c)))) + (@bBuf[ 2]) +&Hfcefa3f8
        ROTATE LEFT d, %S22: d = d + a
        c = c + ((((d)) AND ((b))) OR (((a)) AND (NOT (b)))) + (@bBuf[ 7]) +&H676f02d9
        ROTATE LEFT c, %S23: c = c + d
        b = b + ((((c)) AND ((a))) OR (((d)) AND (NOT (a)))) + (@bBuf[12]) +&H8d2a4c8a
        ROTATE LEFT b, %S24: b = b + c

        ' ROUND 3
        a = a + (((b)) XOR ((c)) XOR ((d))) + (@bBuf[ 5]) +&Hfffa3942
        ROTATE LEFT a, %S31: a = a + b
        d = d + (((a)) XOR ((b)) XOR ((c))) + (@bBuf[ 8]) +&H8771f681
        ROTATE LEFT d, %S32: d = d + a
        c = c + (((d)) XOR ((a)) XOR ((b))) + (@bBuf[11]) +&H6d9d6122
        ROTATE LEFT c, %S33: c = c + d
        b = b + (((c)) XOR ((d)) XOR ((a))) + (@bBuf[14]) +&Hfde5380c
        ROTATE LEFT b, %S34: b = b + c
        '
        a = a + (((b)) XOR ((c)) XOR ((d))) + (@bBuf[ 1]) +&Ha4beea44
        ROTATE LEFT a, %S31: a = a + b
        d = d + (((a)) XOR ((b)) XOR ((c))) + (@bBuf[ 4]) +&H4bdecfa9
        ROTATE LEFT d, %S32: d = d + a
        c = c + (((d)) XOR ((a)) XOR ((b))) + (@bBuf[ 7]) +&Hf6bb4b60
        ROTATE LEFT c, %S33: c = c + d
        b = b + (((c)) XOR ((d)) XOR ((a))) + (@bBuf[10]) +&Hbebfbc70
        ROTATE LEFT b, %S34: b = b + c
        '
        a = a + (((b)) XOR ((c)) XOR ((d))) + (@bBuf[13]) +&H289b7ec6
        ROTATE LEFT a, %S31: a = a + b
        d = d + (((a)) XOR ((b)) XOR ((c))) + (@bBuf[ 0]) +&Heaa127fa
        ROTATE LEFT d, %S32: d = d + a
        c = c + (((d)) XOR ((a)) XOR ((b))) + (@bBuf[ 3]) +&Hd4ef3085
        ROTATE LEFT c, %S33: c = c + d
        b = b + (((c)) XOR ((d)) XOR ((a))) + (@bBuf[ 6]) +&H4881d05
        ROTATE LEFT b, %S34: b = b + c
        '
        a = a + (((b)) XOR ((c)) XOR ((d))) + (@bBuf[ 9]) +&Hd9d4d039
        ROTATE LEFT a, %S31: a = a + b
        d = d + (((a)) XOR ((b)) XOR ((c))) + (@bBuf[12]) +&He6db99e5
        ROTATE LEFT d, %S32: d = d + a
        c = c + (((d)) XOR ((a)) XOR ((b))) + (@bBuf[15]) +&H1fa27cf8
        ROTATE LEFT c, %S33: c = c + d
        b = b + (((c)) XOR ((d)) XOR ((a))) + (@bBuf[ 2]) +&Hc4ac5665
        ROTATE LEFT b, %S34: b = b + c

        ' ROUND 4
        a = a + (((c)) XOR (((b)) OR (NOT (d)))) + (@bBuf[ 0]) +&Hf4292244
        ROTATE LEFT a, %S41: a = a + b
        d = d + (((b)) XOR (((a)) OR (NOT (c)))) + (@bBuf[ 7]) +&H432aff97
        ROTATE LEFT d, %S42: d = d + a
        c = c + (((a)) XOR (((d)) OR (NOT (b)))) + (@bBuf[14]) +&Hab9423a7
        ROTATE LEFT c, %S43: c = c + d
        b = b + (((d)) XOR (((c)) OR (NOT (a)))) + (@bBuf[ 5]) +&Hfc93a039
        ROTATE LEFT b, %S44: b = b + c
        '
        a = a + (((c)) XOR (((b)) OR (NOT (d)))) + (@bBuf[12]) +&H655b59c3
        ROTATE LEFT a, %S41: a = a + b
        d = d + (((b)) XOR (((a)) OR (NOT (c)))) + (@bBuf[ 3]) +&H8f0ccc92
        ROTATE LEFT d, %S42: d = d + a
        c = c + (((a)) XOR (((d)) OR (NOT (b)))) + (@bBuf[10]) +&Hffeff47d
        ROTATE LEFT c, %S43: c = c + d
        b = b + (((d)) XOR (((c)) OR (NOT (a)))) + (@bBuf[ 1]) +&H85845dd1
        ROTATE LEFT b, %S44: b = b + c
        '
        a = a + (((c)) XOR (((b)) OR (NOT (d)))) + (@bBuf[ 8]) +&H6fa87e4f
        ROTATE LEFT a, %S41: a = a + b
        d = d + (((b)) XOR (((a)) OR (NOT (c)))) + (@bBuf[15]) +&Hfe2ce6e0
        ROTATE LEFT d, %S42: d = d + a
        c = c + (((a)) XOR (((d)) OR (NOT (b)))) + (@bBuf[ 6]) +&Ha3014314
        ROTATE LEFT c, %S43: c = c + d
        b = b + (((d)) XOR (((c)) OR (NOT (a)))) + (@bBuf[13]) +&H4e0811a1
        ROTATE LEFT b, %S44: b = b + c
        '
        a = a + (((c)) XOR (((b)) OR (NOT (d)))) + (@bBuf[ 4]) +&Hf7537e82
        ROTATE LEFT a, %S41: a = a + b
        d = d + (((b)) XOR (((a)) OR (NOT (c)))) + (@bBuf[11]) +&Hbd3af235
        ROTATE LEFT d, %S42: d = d + a
        c = c + (((a)) XOR (((d)) OR (NOT (b)))) + (@bBuf[ 2]) +&H2ad7d2bb
        ROTATE LEFT c, %S43: c = c + d
        b = b + (((d)) XOR (((c)) OR (NOT (a)))) + (@bBuf[ 9]) +&Heb86d391
        ROTATE LEFT b, %S44: b = b + c

        @tContext.lState(0) = @tContext.lState(0) + a
        @tContext.lState(1) = @tContext.lState(1) + b
        @tContext.lState(2) = @tContext.lState(2) + c
        @tContext.lState(3) = @tContext.lState(3) + d

END SUB

'-------------------------------------------------------------------------------------------
' Update context TO reflect the concatenation of another buffer full of bytes
'-------------------------------------------------------------------------------------------
SUB MD5_Update(pCtx AS DWORD, sBuf AS STRING, BYVAL dwLen AS DWORD)

        LOCAL t            AS DWORD       '
        LOCAL p            AS BYTE PTR    '
        LOCAL ctx          AS MD5_CTX PTR '
        LOCAL Temp         AS STRING PTR * 300
        LOCAL bBuf         AS STRING PTR * 300
        LOCAL dBuf         AS STRING     * 300

        dBuf = sBuf        ' flexiblen Strin umwrappem
        ctx = pCtx
        bBuf = VARPTR(dBuf)

        ' Update bitcount
        t = @ctx.lCount(0)
        @ctx.lCount(0) = t + MD5_ShiftLeft(dwLen, 3)
        IF @ctx.lCount(0) < t THEN
            INCR @ctx.lCount(1)                ' carry from low to high
        END IF

        @ctx.lCount(1) = @ctx.lCount(1) + MD5_ShiftRight(dwLen, 29)

        t = (MD5_ShiftRight(t, 3) AND &H3F)    ' Bytes already IN shsInfo -> DATA

        ' handle any leading odd-sized chunks
        IF ISTRUE(t) THEN
            p = VARPTR(@ctx.bBuf)
            p = p + t
            t = 64 - t
            IF dwLen < t THEN
                CALL MD5_MoveMemory(p, bBuf, dwLen)
                EXIT SUB
            END IF
            CALL Md5_MoveMemory(p, bBuf, t)
            CALL MD5_Transform(ctx, VARPTR(@ctx.bBuf))
            bBuf = bBuf + t
            dwLen = dwLen - t
        END IF

        ' Process DATA IN 64-BYTE chunks
        DO WHILE (dwLen => 64)
            CALL MD5_MoveMemory(VARPTR(@ctx.bBuf), bBuf, 64)
            CALL MD5_Transform(ctx, VARPTR(@ctx.bBuf) )
            bBuf = bBuf + 64
            dwLen = dwLen - 64
        LOOP
        'handle any remaining bytes of data
        CALL MD5_MoveMemory(VARPTR(@ctx.bBuf), bBuf, dwLen)

END SUB

'-------------------------------------------------------------------------------------------
' Final wrapup - pad TO 64-BYTE boundary WITH the BIT pattern
' 1 0* (64-BIT count of bits processed, MSB-first)
'-------------------------------------------------------------------------------------------
SUB MD5_Final(dwDigest AS STRING * 16, pCtx AS DWORD)

        LOCAL lCount       AS DWORD       '
        LOCAL p            AS BYTE PTR    '
        LOCAL bBuffer      AS BYTE PTR    '
        LOCAL ctx          AS MD5_CTX PTR '

        ctx = pCtx
        bBuffer = VARPTR(@ctx.bBuf)

        ' Compute number of bytes MOD 64
        lCount = (MD5_ShiftRight(@ctx.lCount(0), 3) AND &H3F)

        ' set the first char of padding to &H80.  This is safe since there is
        ' always AT least one BYTE free
        p = VARPTR(@ctx.bBuf)
        p = p + lCount
        @p = &H80
        INCR p

        ' Bytes of padding needed TO make 64 bytes
        lCount = 64 - 1 - lCount

        ' Pad out TO 56 MOD 64
        IF lCount < 8 THEN
            ' two lots of padding: pad the first block to 64 bytes
            ' PowerBASIC fllt ASCIIZ immer mit Zero auf
            CALL MD5_FillMemory(p, lCount, 0)
            CALL MD5_Transform(ctx, VARPTR(@ctx.bBuf))
            CALL MD5_FillMemory(VARPTR(@ctx.bBuf), 56, 0)
        ELSE
            ' pad block to 56 bytes
            CALL MD5_FillMemory(p, (lCount - 8), 0)
        END IF

        ' append length IN bits and transform
        CALL MD5_MoveMemory(VARPTR(@bBuffer[14 * SIZEOF(lCount)]), VARPTR(@ctx.lCount(0)), SIZEOF(lCount))
        CALL MD5_MoveMemory(VARPTR(@bBuffer[15 * SIZEOF(lCount)]), VARPTR(@ctx.lCount(1)), SIZEOF(lCount))
        CALL MD5_Transform(ctx, VARPTR(@ctx.bBuf))
        'CALL MD5_MoveMemory(dwDigest, VARPTR(@ctx.lState(0)), 16)
        dwDigest = MKL$(@ctx.lState(0)) + MKL$(@ctx.lState(1)) + MKL$(@ctx.lState(2)) + MKL$(@ctx.lState(3))

        'CALL FillMemory(ctx, SIZEOF(@ctx), 0)
        @ctx.lState(1) = 0                ' alte Datenfelder lschen
        @ctx.lState(2) = 0                '
        @ctx.lState(3) = 0                '
        @ctx.lState(4) = 0                '
        @ctx.lCount(1) = 0                '
        @ctx.lCount(2) = 0                '
        @ctx.bBuf      = STRING$(64, 0)   ' Ersatz

END SUB

'-------------------------------------------------------------------------------------------
' Creates Native-String
'-------------------------------------------------------------------------------------------
FUNCTION MD5_String(BYVAL dwContext AS DWORD) AS STRING

        ' returns the MD5 digest as a hex string
        LOCAL pContext     AS MD5_CTX PTR '
        LOCAL szDigest     AS STRING * 16 '

        pContext = dwContext

        CALL MD5_Final(szDigest, pContext)

        FUNCTION = szDigest

END FUNCTION

'-------------------------------------------------------------------------------------------
' Creates 32Byte Hex-String
'-------------------------------------------------------------------------------------------
FUNCTION MD5_Digest(BYVAL dwContext AS DWORD) AS STRING

        ' returns the MD5 digest as a hex string
        LOCAL sHex         AS STRING      '
        LOCAL pContext     AS MD5_CTX PTR '
        LOCAL szDigest     AS STRING * 16 '
        LOCAL bDigest      AS BYTE PTR    '
        LOCAL sCheck       AS STRING      '
        LOCAL i            AS LONG        '

        bDigest = VARPTR(szDigest)
        pContext = dwContext

        CALL MD5_Final(szDigest, pContext)

        IF bDigest <> %NULL THEN
            FOR i& = 0 TO 15
                sHex = sHex + HEX$(@bDigest[i&],2)
            NEXT i&
        END IF
        FUNCTION = LCASE$(sHex)

END FUNCTION


'-------------------------------------------------------------------------------------------
' Routines for MD5 (used for SASL/Digest-MD5, RFC 1321)
'-------------------------------------------------------------------------------------------
FUNCTION Get_String_MD5(BYVAL sText AS STRING) AS STRING

        LOCAL MD5_Ctx_Digest AS MD5_CTX     '

        CALL MD5_Init(VARPTR(MD5_Ctx_Digest))
        CALL MD5_Update(VARPTR(MD5_Ctx_Digest), sText, LEN(sText) )
        FUNCTION = MD5_String(VARPTR(MD5_Ctx_Digest))

END FUNCTION

'-------------------------------------------------------------------------------------------
' Routines for MD5 (used for APOP/Digest-MD5, RFC 1321)
'-------------------------------------------------------------------------------------------
FUNCTION Get_Digest_MD5(BYVAL sText AS STRING) AS STRING

        LOCAL MD5_Ctx_Digest AS MD5_CTX     '

        CALL MD5_Init(VARPTR(MD5_Ctx_Digest))
        CALL MD5_Update(VARPTR(MD5_Ctx_Digest), sText, LEN(sText) )
        FUNCTION = MD5_Digest(VARPTR(MD5_Ctx_Digest))

END FUNCTION

'-------------------------------------------------------------------------------------------
' Routines for HMAC-MD5 (used for CRAM-MD5, RFC 2104)
'-------------------------------------------------------------------------------------------
FUNCTION Get_CRAM_MD5(BYVAL sText AS STRING, BYVAL sKey AS STRING) AS STRING

        LOCAL sKeyLen      AS LONG        '
        LOCAL MD5_Context  AS MD5_CTX
        LOCAL Temp_Ctx     AS MD5_CTX
        LOCAL ipad         AS STRING          ' inner padding - key XORd WITH ipad
        LOCAL opad         AS STRING          ' outer padding - key XORd WITH opad
        LOCAL p_ipad       AS BYTE PTR        '
        LOCAL p_opad       AS BYTE PTR
        LOCAL sDigest      AS STRING          ' byte array with digest
        LOCAL szDigest     AS STRING * 16
        LOCAL sTk          AS STRING * 16     ' unsigned char tk[16]
        LOCAL i            AS LONG            '

        ipad    = STRING$(64, 0)              ' Strings mssen immer gefllt sein!
        opad    = STRING$(64, 0)
        sDigest = STRING$(16, 0)

        '* IF key is longer than 64 bytes RESET it TO key=MD5(key)
        IF LEN(sKey) > 64 THEN
            CALL MD5_Init(VARPTR(Temp_Ctx))
            CALL MD5_Update(VARPTR(Temp_Ctx), sKey, LEN(sKey))
            CALL MD5_Final(sTk, VARPTR(Temp_Ctx))
            sKey     = sTk
            sKeyLen  = 16
            ipad     = sTk + STRING$(64-LEN(sTk), 0)
            opad     = sTk + STRING$(64-LEN(sTk), 0)
        ELSE
            ipad     = sKey + STRING$(64-LEN(sKey), 0)
            opad     = sKey + STRING$(64-LEN(sKey), 0)
        END IF

        ' PowerBASIC fllt ASCIIZ immer mit Zero auf, deshalb bei dynamischen Strings per Hand nachhelfen!
        ' CALL FillMemory(VARPTR(k_ipad), LEN(k_ipad), 0)   ' bzero( k_ipad, SIZEOF k_ipad);
        ' CALL FillMemory(VARPTR(k_opad), LEN(k_opad), 0)   ' bzero( k_opad, SIZEOF k_opad);

        'CALL MoveMemory(VARPTR(k_ipad), VARPTR(szKey), szKeyLen) ' bcopy( key, k_ipad, key_len);
        'CALL MoveMemory(VARPTR(k_opad), VARPTR(szKey), szKeyLen) ' bcopy( key, k_opad, key_len);

        '* ipad is the BYTE 0x36 repeated 64 times
        '* opad is the BYTE 0x5c repeated 64 times
        '* AND text is the DATA being protected
        '*
        '* start out by storing key IN pads
        '* XOR key WITH ipad AND opad values

        p_ipad = STRPTR(ipad)
        p_opad = STRPTR(opad)

        FOR i& = 0 TO 63
             @p_ipad[i] = @p_ipad[i] XOR &h36
             @p_opad[i] = @p_opad[i] XOR &h5c
        NEXT i%

        '*
        '* perform inner MD5
        '*
        CALL MD5_Init(VARPTR(MD5_Context))                       '* init context FOR 1st
        CALL MD5_Update(VARPTR(MD5_Context), ipad, 64)           '* start WITH inner pad
        CALL MD5_Update(VARPTR(MD5_Context), sText, LEN(sText))  '* THEN text OF datagram
        CALL MD5_Final(szDigest, VARPTR(MD5_Context))            '* finish UP 1st pass
        sDigest = szDigest
        '*
        '* perform outer MD5
        '*
        CALL MD5_Init(VARPTR(MD5_Context))                       '* init context FOR 2nd
        CALL MD5_Update(VARPTR(MD5_Context), opad, 64)           '* start WITH outer pad
        CALL MD5_Update(VARPTR(MD5_Context), sDigest, 16)        '* THEN results OF 1st

        FUNCTION = MD5_Digest(VARPTR(MD5_Context))

END FUNCTION
